from django.http import JsonResponse
from django.urls import reverse
from django.views import View
from extra_views import CreateWithInlinesView, InlineFormSet
from extra_views import UpdateWithInlinesView

from .models import CSchedules, CSchedulesDays
from .source import DAY_CHOICE, VAR_BY_COUNTRY
from .forms import CSchedulesForm


class CSchedulesDaysInline(InlineFormSet):
    model = CSchedulesDays
    fields = ['day_number', 'start_time', 'end_time']
    extra = 1
    form_class = CSchedulesForm

    def get_formset_kwargs(self):
        kwargs = super().get_formset_kwargs()

        kwargs['data'] = kwargs['data'].dict()

        if self.request.method == 'POST':
            kwargs['save_as_new'] = True

        return kwargs


class BaseCSchedulesView:
    model = CSchedules
    inlines = [CSchedulesDaysInline]
    fields = ['title', 'timezone', 'timezone_place', 'use_work_weekend', 'use_holiday']

    def get_context_data(self, **kwargs):
        ctx = super(BaseCSchedulesView, self).get_context_data(**kwargs)
        ctx['days'] = DAY_CHOICE
        ctx['hour_range'] = range(0, 24)
        return ctx

    def get_success_url(self):
        return reverse('cs_update', args=(self.object.id,))

    def get_form_kwargs(self):
        """
        Returns the keyword arguments for instantiating the form.
        """
        kwargs = {
            'initial': self.get_initial(),
            'prefix': self.get_prefix(),
        }

        if self.request.method in ('POST', 'PUT'):
            kwargs.update({
                'data': self.request.POST.dict(),
                'files': self.request.FILES,
            })
        return kwargs


class CreateCSchedulesView(BaseCSchedulesView, CreateWithInlinesView):
    template_name = 'c_schedules/create.html'

    def post(self, request, *args, **kwargs):
        return super().post(request, *args, **kwargs)

    def forms_valid(self, form, inlines):
        resp = super().forms_valid(form, inlines)

        # attach owner if exist
        user = self.request.user
        self.object.owner = user if user.is_authenticated() else None
        self.object.save()

        return resp


class UpdateCSchedulesView(BaseCSchedulesView, UpdateWithInlinesView):
    template_name = 'c_schedules/update.html'

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()

        # delete old CSchedulesDays
        self.object.days.all().delete()

        return super(UpdateCSchedulesView, self).post(request, *args, **kwargs)


class LoadTimezonePlaceView(View):
    http_method_names = ['get']

    def get(self, request, *args, **kwargs):
        area = VAR_BY_COUNTRY.get(kwargs.get('timezone_value'), [])
        return JsonResponse({
            'results': [''] + list(area.keys())
        })



from django.conf.urls import url

from django.conf import settings
from .views import CreateCSchedulesView, UpdateCSchedulesView, LoadTimezonePlaceView

urlpatterns = [
    url(r'^create/$', CreateCSchedulesView.as_view(), name='cs_create'),
    url(r'^update/(?P<pk>[0-9]+)/$', UpdateCSchedulesView.as_view(), name='cs_update'),
    url(r'^timezone_place/(?P<timezone_value>\w+)/$', LoadTimezonePlaceView.as_view(), name='load_timezone_place'),
]

if getattr(settings, 'USE_ONLY_REST_URLS', False):
    from .rest.v1.urls import rest_urlpatterns

    urlpatterns = rest_urlpatterns
elif getattr(settings, 'USE_WITH_REST_URLS', False):
    from .rest.v1.urls import rest_urlpatterns

    urlpatterns += rest_urlpatterns



from django import forms
from .models import CSchedules
from .source import VAR_BY_COUNTRY


class CSchedulesForm(forms.ModelForm):
    def clean_timezone(self):
        tz = self.cleaned_data.get('timezone')

        if tz not in VAR_BY_COUNTRY.keys():
            raise forms.ValidationError('Некорректное значение часового пояса')

        return tz

    def clean_timezone_place(self):
        tz = self.cleaned_data.get('timezone')
        tz_place = self.cleaned_data.get('timezone_place')

        if tz_place not in VAR_BY_COUNTRY[tz]:
            raise forms.ValidationError('Некорректное значение местоположения')

        return tz_place

    class Meta:
        model = CSchedules
        fields = ('title', 'timezone', 'timezone_place', 'use_work_weekend', 'use_holiday')



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-04-04 11:52
from __future__ import unicode_literals

import datetime
import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('c_schedules', '0002_auto_20180404_1449'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cschedulesdays',
            name='end_time',
            field=models.TimeField(validators=[django.core.validators.MinValueValidator(datetime.time(1, 0)), django.core.validators.MaxValueValidator(datetime.time(23, 59))], verbose_name='Конец'),
        ),
        migrations.AlterField(
            model_name='cschedulesdays',
            name='start_time',
            field=models.TimeField(validators=[django.core.validators.MinValueValidator(datetime.time(0, 0), django.core.validators.MaxValueValidator(datetime.time(22, 0)))], verbose_name='Старт'),
        ),
    ]



from django.contrib import messages
from rest_framework.viewsets import ModelViewSet

from .serializers import CShedulesSerializer
from ...models import CSchedules


class CSchedulesViewSet(ModelViewSet):
    serializer_class = CShedulesSerializer

    def get_queryset(self):
        if self.request.user.is_authenticated():
            return self.request.user.c_shedules_list.all()
        else:
            return CSchedules.objects.all()

    def perform_create(self, serializer):
        instance = serializer.save()

        messages.add_message(
            self.request, messages.SUCCESS,
            'Schedules with title "{0}" success created'.format(instance.title)
        )

    def perform_update(self, serializer):
        instance = serializer.save()

        messages.add_message(
            self.request, messages.SUCCESS,
            'Schedules with title "{0}" success updated'.format(instance.title)
        )

    def perform_destroy(self, instance):
        title = instance.title

        super().perform_destroy(instance)

        messages.add_message(
            self.request, messages.SUCCESS,
            'Schedules with title "{0}" success deleted'.format(title)
        )



from django.conf import settings
from django.core.validators import MinValueValidator, MaxValueValidator
from django.db import models
from django.utils.dateparse import parse_time

from .source import RUSSIA, CIS, OTHER_COUNTRIES, DAY_CHOICE


class CSchedules(models.Model):
    TIMEZONE_NAME_CHOICES = (
        (RUSSIA, 'Россия'),
        (CIS, 'СНГ'),
        (OTHER_COUNTRIES, 'Страны мира'),
    )

    title = models.CharField(verbose_name='Заголовок расписания', max_length=100)
    owner = models.ForeignKey(
        settings.AUTH_USER_MODEL, related_name='c_shedules_list', verbose_name='Создатель',
        blank=True, null=True
    )

    timezone = models.CharField(verbose_name='Часовой пояс', max_length=3, choices=TIMEZONE_NAME_CHOICES)
    timezone_place = models.CharField(verbose_name='Местоположение', max_length=100)

    use_work_weekend = models.BooleanField(verbose_name='Учитывать рабочие выходные', default=True)
    use_holiday = models.BooleanField(verbose_name='Учитывать праздничные дни', default=False)

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        verbose_name = 'Планировщик расписания'
        verbose_name_plural = 'Список планировщиков расписаний'
        ordering = ['-created_at']

    def __str__(self):
        return self.title


class CSchedulesDays(models.Model):
    c_schedules = models.ForeignKey(CSchedules, related_name='days', verbose_name='Планировщик')

    day_number = models.IntegerField(verbose_name='Номер дня', validators=[MinValueValidator(1), MaxValueValidator(7)])

    start_time = models.TimeField(verbose_name='Старт', validators=[
        MinValueValidator(parse_time("00:00"), MaxValueValidator(parse_time("23:00")))])
    end_time = models.TimeField(
        verbose_name='Конец', validators=[
            MinValueValidator(parse_time("01:00")),
            MaxValueValidator(parse_time("23:59"))]
    )

    @property
    def day_name(self):
        return dict(DAY_CHOICE).get(self.day_number)

    class Meta:
        verbose_name = 'День в расписании'
        verbose_name_plural = 'Список дней в расписании'
        unique_together = ['c_schedules', 'day_number', 'start_time', 'end_time']
        ordering = ['day_number']

    def __str__(self):
        return "{0} - start:{1}; end:{2}".format(self.day_number, self.start_time, self.end_time)



from django.conf.urls import url
from rest_framework.urlpatterns import format_suffix_patterns

from .views import CSchedulesViewSet

cs_list = CSchedulesViewSet.as_view({
    'get': 'list',
    'post': 'create'
})

cs_detail = CSchedulesViewSet.as_view({
    'put': 'update',
    'delete': 'destroy'
})

urlpatterns = [
    url(r'^api_list/$', cs_list),
    url(r'^api_list/(?P<pk>[0-9]+)/$', cs_detail),
]

rest_urlpatterns = format_suffix_patterns(urlpatterns)



from django.contrib.auth.models import AnonymousUser
from rest_framework import serializers

from ...models import CSchedules, CSchedulesDays
from ...source import VAR_BY_COUNTRY


class CShedulesDaysSerializer(serializers.ModelSerializer):
    start_time = serializers.TimeField(format="%H:%M", required=True)
    end_time = serializers.TimeField(format="%H:%M", required=True)
    day_name = serializers.SerializerMethodField(read_only=True)

    def get_day_name(self, obj):
        return obj.day_name

    def validate(self, attrs):
        if attrs['start_time'] >= attrs['end_time']:
            raise serializers.ValidationError('end time should be longer than star time')

        return attrs

    class Meta:
        model = CSchedulesDays
        fields = ('id', 'day_number', 'day_name', 'start_time', 'end_time')
        read_only_fields = ('id',)


class CShedulesSerializer(serializers.ModelSerializer):
    days = CShedulesDaysSerializer(many=True, required=True)
    timezone_offset = serializers.SerializerMethodField(read_only=True)
    timezone_name = serializers.SerializerMethodField(read_only=True)

    def get_timezone_name(self, obj):
        return obj.get_timezone_display()

    def get_timezone_offset(self, obj):
        tz = obj.timezone
        tz_place = obj.timezone_place
        try:
            return VAR_BY_COUNTRY[tz][tz_place]
        except KeyError:
            return 'incorrent value'

    def validate_title(self, title):
        user = self._get_current_user()
        if user.is_authenticated():
            try:
                user.c_shedules_list.get(title=title)
                raise serializers.ValidationError(
                    'Schedule with title: :{0}" from the user: "{1}" already exist'.format(title, user)
                )
            except CSchedules.DoesNotExist:
                pass

        return title

    def validate_days(self, days):
        if not days:
            raise serializers.ValidationError('Days should not be empty')

        for day in days:
            pass

        return days

    def validate(self, attrs):
        timezone = attrs.get('timezone')
        current_place = attrs.get('timezone_place')
        timezone_places = VAR_BY_COUNTRY.get(timezone)

        if current_place not in timezone_places.keys():
            raise serializers.ValidationError('selected timezone place does not belong to selected timezone')

        return attrs

    class Meta:
        model = CSchedules
        fields = (
            'id', 'title', 'timezone', 'timezone_name', 'timezone_place',
            'timezone_offset', 'use_work_weekend',
            'use_holiday', 'days'
        )
        read_only_fields = ('id',)

    def create(self, validated_data):
        current_user = self._get_current_user()
        days = validated_data.pop('days', [])

        validated_data['owner'] = current_user if current_user.is_authenticated() else None

        instance = super().create(validated_data)

        self._save_days(instance, days)

        return instance

    def update(self, instance, validated_data):
        days = validated_data.pop('days', [])

        instance = super().update(instance, validated_data)

        self._save_days(instance, days)

        return instance

    def _save_days(self, instance, days):
        # remove old schedule
        instance.days.all().delete()

        # create or update new schedule
        for day_data in days:
            try:
                day = instance.days.get(day_number=day_data['day_number'], end_time=day_data['end_time'])
                self._update_fields(day, day_data)
            except CSchedulesDays.DoesNotExist:
                day = instance.days.create(**day_data)
            day.save()

    def _update_fields(self, instance, data):
        [setattr(instance, k, v) for k, v in data.items()]

    def _get_current_user(self):
        try:
            return self.context['request'].user
        except KeyError:
            return AnonymousUser()



import os
from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-cschedules',
    version='0.0.8',
    packages=['c_schedules'],
    include_package_data=True,
    license='BSD License',
    description='Very simple calendar schedule',
    long_description=README,
    url='http://git.dpsoft.org/puvka/cshedules.git',
    author='puvka',
    author_email='lendo.fox@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=[
        'Django>=1.10.8,<=1.11.1',
        'django-extra-views',
        'requests==2.18.4',
    ]
)



from django.contrib import admin
from .models import CSchedulesDays, CSchedules
from .source import DAY_CHOICE


class CShedulesDaysInline(admin.TabularInline):
    model = CSchedulesDays
    extra = 3


@admin.register(CSchedules)
class CShedulesAdmin(admin.ModelAdmin):
    list_display = ['title', 'owner', 'use_work_weekend', 'use_holiday', 'updated_at', 'day_count']
    list_filter = ['title', 'use_holiday', 'use_work_weekend']
    search_fields = ['title', 'owner', 'use_work_weekend', 'use_holiday', 'day_count']
    inlines = [CShedulesDaysInline]

    add_form_template = 'admin/c_shedules/change_form.html'
    change_form_template = 'admin/c_shedules/change_form.html'

    def day_count(self, obj):
        return obj.days.count()

    day_count.short_description = 'Количество дней'

    def changeform_view(self, request, object_id=None, form_url='', extra_context=None):
        extra_context = {
            'days': DAY_CHOICE,
            'hour_range': range(0, 24)
        }
        return super().changeform_view(request, object_id=object_id, form_url=form_url, extra_context=extra_context)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-04-03 12:07
from __future__ import unicode_literals

import datetime
from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='CSchedules',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=100, verbose_name='Заголовок расписания')),
                ('timezone', models.CharField(choices=[('ru', 'Россия'), ('cis', 'СНГ'), ('others', 'Страны мира')], max_length=3, verbose_name='Часовой пояс')),
                ('timezone_place', models.CharField(max_length=100, verbose_name='Местоположение')),
                ('use_work_weekend', models.BooleanField(default=True, verbose_name='Учитывать рабочие выходные')),
                ('use_holiday', models.BooleanField(default=False, verbose_name='Учитывать праздничные дни')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('owner', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='c_shedules_list', to=settings.AUTH_USER_MODEL, verbose_name='Создатель')),
            ],
            options={
                'verbose_name': 'Планировщик расписания',
                'verbose_name_plural': 'Список планировщиков расписаний',
                'ordering': ['-created_at'],
            },
        ),
        migrations.CreateModel(
            name='CSchedulesDays',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('day_number', models.IntegerField(validators=[django.core.validators.MinValueValidator(1), django.core.validators.MaxValueValidator(7)], verbose_name='Номер дня')),
                ('start_time', models.TimeField(validators=[django.core.validators.MinValueValidator(datetime.time(0, 0), django.core.validators.MaxValueValidator(datetime.time(23, 59)))], verbose_name='Старт')),
                ('end_time', models.TimeField(verbose_name='Конец')),
                ('c_schedules', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='days', to='c_schedules.CSchedules', verbose_name='Планировщик')),
            ],
            options={
                'verbose_name': 'День в расписании',
                'verbose_name_plural': 'Список дней в расписании',
                'ordering': ['day_number'],
            },
        ),
        migrations.AlterUniqueTogether(
            name='cschedulesdays',
            unique_together=set([('c_schedules', 'day_number', 'start_time', 'end_time')]),
        ),
    ]
